/* Plugin: MedArmory-PRO
 * File: utilfunctions.cpp
 * Author: Prodigy (jcouzens@uhfco.net)
 * Date: March 30, 2003
 * Info: util functions
 * Updated: April 5, 2003 : Function Add: ability to see if structure is built
*/

#include <extdll.h>	//always
#include <string.h>	//to prevent linux errors while processing meta_api.h
#include <sdk_util.h>	//pre-defined function definitions saves us a lot of work
#include <meta_api.h>	// Plugin_info, etc
#include <cbase.h>
#include <player.h>

#include "plugin.h"
#include "utilfunctions.h"

DLL_GLOBAL const Vector     g_vecZero = Vector(0,0,0);

int gmsgTextMsg = 0;
int gmsgSayText = 0;

int global_time = 0;
float global_decay = 0;
float global_radius = 0;

//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG: ( timestamp ) < message >
//=========================================================
void UTIL_LogPrintf(char *fmt, ...)
{
	va_list			argptr;
	static char		string[1024];
	
	va_start (argptr, fmt);
	vsprintf (string, fmt, argptr);
	va_end   (argptr);

	// Print to server console
	ALERT(at_logged, "%s", string);
}


// returns a CBaseEntity pointer to a player by index.  Only returns if the player is spawned and connected
// otherwise returns NULL
// Index is 1 based
CBaseEntity	*UTIL_PlayerByIndex(int playerIndex)
{
	CBaseEntity *pPlayer = NULL;

	if (playerIndex > 0 && playerIndex <= gpGlobals->maxClients)
	{
		edict_t *pPlayerEdict = INDEXENT(playerIndex);
		if (pPlayerEdict && !pPlayerEdict->free)
		{
			pPlayer = CBaseEntity::Instance(pPlayerEdict);	
		}
	}
	
	return pPlayer;
}

int UTIL_GetPlayerIndex(CBasePlayer *pPlayer)
{
	if(pPlayer == NULL) return 0;
	int counter;
	for(counter = 1; counter <= gpGlobals->maxClients; counter++)
	{
		if(pPlayer == UTIL_PlayerByIndex(counter))
			return counter;
	}
	return 0;
}


CBaseEntity *UTIL_FindEntityByString(CBaseEntity *pStartEntity, const char *szKeyword, const char *szValue)
{
	edict_t	*pentEntity;

	if (pStartEntity)
		pentEntity = pStartEntity->edict();
	else
		pentEntity = NULL;

	pentEntity = FIND_ENTITY_BY_STRING(pentEntity, szKeyword, szValue);

	if (!FNullEnt(pentEntity))
		return CBaseEntity::Instance(pentEntity);
	return NULL;
}

CBaseEntity *UTIL_FindEntityInSphere(CBaseEntity *pStartEntity, const Vector &vecCenter, float flRadius)
{
	edict_t *pentEntity;

	if (pStartEntity)
	{
		pentEntity = pStartEntity->edict();
	}
  else
	{
    pentEntity = NULL;
	}

	pentEntity = FIND_ENTITY_IN_SPHERE( pentEntity, vecCenter, flRadius);

	if (!FNullEnt(pentEntity))
	{
		return CBaseEntity::Instance(pentEntity);
	}

	return NULL;
}


void UTIL_ParticleBeam_All(CBaseEntity *client, Vector origin, unsigned char colorRed, unsigned char colorGreen, unsigned char colorBlue, unsigned char colorBright, unsigned char displaySpeed)
{
	MESSAGE_BEGIN(MSG_ONE, SVC_TEMPENTITY, NULL, client->edict());
	WRITE_BYTE(TE_BEAMTORUS);
	WRITE_COORD(origin.x);
	WRITE_COORD(origin.y);
	WRITE_COORD(origin.z + 5);
	WRITE_COORD(origin.x);
	WRITE_COORD(origin.y);
	WRITE_COORD(origin.z + 256);
	WRITE_SHORT(PRECACHE_MODEL("sprites/laserbeam.spr"));
	WRITE_BYTE(0);              // startframe
	WRITE_BYTE(0);              // framerate
	WRITE_BYTE(5);              // life
	WRITE_BYTE(1);              // width
	WRITE_BYTE(0);              // noise
	WRITE_BYTE(colorRed);       // R
	WRITE_BYTE(colorGreen);     // G
	WRITE_BYTE(colorBlue);      // B
	WRITE_BYTE(colorBright);    // brightness
	WRITE_BYTE(displaySpeed);   // speed
	MESSAGE_END();
}

void UTIL_DynamicLight_All(Vector origin, unsigned char colorRed, unsigned char colorGreen, unsigned char colorBlue)
{
	MESSAGE_BEGIN(MSG_BROADCAST, SVC_TEMPENTITY);
	WRITE_BYTE(TE_DLIGHT);
	WRITE_COORD(origin.x);	      // X
	WRITE_COORD(origin.y);	      // Y
	WRITE_COORD(origin.z);	      // Z
	WRITE_BYTE((int)global_radius);			      // radius * 0.1 (10)
	WRITE_BYTE(colorRed);		  // r
	WRITE_BYTE(colorGreen);		  // g
	WRITE_BYTE(colorBlue);		  // b
	WRITE_BYTE(global_time);			      // time * 10 (10)
	WRITE_BYTE((int)global_decay);			      // decay * 0.1 (0)
	MESSAGE_END();
}

int UTIL_EntityIsBuilt(CBaseEntity *pentEntity)
{
	edict_t	*pent;

	pent = pentEntity->edict();

	/* something odd I've found, is that v.health is not a number to be taken seriously,
	 * Although it maxes at 2400 once an armory is built, if you damage it, you will not
	 * be able to return it to 2400.  Instead, it tracks the real health (1000) inside of
	 * v.fuser2 (for BASIC armories).
	*/

	// we've found either an armory (fuser1=1000) or an adv armory (classname = team_advarmory)
	// doing it this way because you can't have an advanced armory, without first having a
	// sucessfully built basic armory.  Also because when an armory goes advanced, fuser changes
	// back to 0.
		
	if(pent->v.fuser1 == 1000 || FStrEq(STRING(pent->v.classname), "team_advarmory"))
	{
		return 1;
	}

	return 0;
}

unsigned short FixedUnsigned16(float value, float scale)
{
	int output;

	output = (int)(value * scale);
	if (output < 0)
		output = 0;
	if (output > 0xFFFF)
		output = 0xFFFF;

	return (unsigned short)output;
}

short FixedSigned16(float value, float scale)
{
	int output;

	output = (int)(value * scale);

	if (output > 32767)
		output = 32767;

	if (output < -32768)
		output = -32768;

	return (short)output;
}

int GetPlayerClass(edict_t *pEntity)
{
	// Entity must be on a team
	if(pEntity->v.team)
	{
		char *infobuffer;
		char model[32];

		infobuffer = (*g_engfuncs.pfnGetInfoKeyBuffer)(pEntity);
		strcpy(model, (g_engfuncs.pfnInfoKeyValue(infobuffer, "model")));

		if (!strcmp(model,   "alien1")) return 1;
		else if(!strcmp(model, "alien2")) return 2;
		else if(!strcmp(model, "alien3")) return 3;
		else if(!strcmp(model, "alien4")) return 4;
		else if(!strcmp(model, "alien5")) return 5;
		else if(!strcmp(model, "gestate")) return 6;
		else if(!strcmp(model, "soldier") && pEntity->v.body == 0) return 7;//Normal
		else if(!strcmp(model, "soldier") && pEntity->v.body == 1) return 8;//Jetpack
		else if(!strcmp(model, "heavy")) return 9;
		else if(!strcmp(model, "commander")) return 10;
	}
	return 0;
}
